import dom from "./elements/Dom.js";
import Request from "./Request.js";

const template = {
	box: () => `
	<div id="request-box" class="vbox">
		<div class="vbox box">
			<div class="row header">Name</div>
			<div class="row content">
				<ul id="requests" class="listing">
				</ul>
			</div>
		</div>
		<div id="summary-bar">Number of requests: 1</div>
	</div>
	<div id="data-box" class="tabs">
		<div id="tab-box">
		</div>
	</div>
	`,
}

export default class RequestList {

	constructor() {
		this.rows = {};

		this.content = dom.create(template.box());
		this.ul = this.content.getElementById('requests');
		this.requestTabs = this.content.getElementById('tab-box');
	}

	get() {
		return this.content;
	}

	show(row) {
		// console.log('requestlist click', row);
		let request_id = row.dataset['request_id'] || "0";
		if (!(request_id in this.rows)) {
			return console.log("unknown request", request_id);
		}

		for (let id in this.rows) {
			// let id = this.rows[i].li.dataset['request_id'];
			if (id != request_id && this.rows[id].li.classList.contains("focus")) {
				this.rows[id].li.classList.remove("focus");
				this.rows[id].request.hide();
			}
		}
		this.rows[request_id].li.classList.add("focus");
		this.rows[request_id].request.show();
		this.request_id = request_id;
	}

	// route incoming event data to matching Request
	add(data) {
		let request_id = data.request_id || "0";
		// console.log("requestList::add", request_id);
		let li = false;
		// if new request_id then create new Request instance first
		if (request_id in this.rows === false) {
			// create new row in request list
			li = dom.li({ text: '-', data: { request_id: request_id }, appendTo: this.ul });
			let req = new Request(request_id, li);

			this.rows[request_id] = {
				li: li,
				request: req
			}
			req.hide();
			this.requestTabs.appendChild(req.get());
			this.updateCounter();
			// console.log('append request content', req.get());
		}
		// console.log(this.rows);
		// console.log("Event", data.type);
		try {
			this.rows[request_id].request.add(data);
			//if new Request is the first one then maybe it must be activated?
			if (this.ul.childElementCount == 1 && li) {
				// console.log('RequestList::add -- show first');
				this.show(li);
			}
		}
		catch (error) {
			console.error("unknown event", data.type, error);
		}
	}

	clearAll() {
		for (let req in this.rows) {
			this.rows[req].request.delete();
		}
		this.rows = {};
		this.ul.innerHTML = "";
		this.requestTabs.innerHTML = "";
		this.updateCounter();
	}

	updateCounter() {
		let summary = document.getElementById("summary-bar");
		let count = Object.keys(this.rows).length;
		summary.innerText = count ? "Number of requests: " + count : '';
	}
}