export default class ZRaySocket {
	constructor(tabId, url) {
		this.tabId = tabId;
		this.url = url;
		this.state = WebSocket.CONNECTING;

		this.reconnectInterval = 5000;
		this.reconnects = 0;
		this.interval = 2000;
		this.closing = false;
		this.timedOut = false;
		this.aliveTimer = false;
	}

	onopen(event) { console.log("CONNECTED", event); };
	onclose(event) { console.log("DISCONNECTED"); };
	onmessage(event) { console.log("MESSAGE", event.data); };
	onerror(event) { console.error("ERROR", event); };

	connect(reconnect) {
		if (this.closing) return;
		this.ws = new WebSocket(this.url);

		var timeout = setTimeout(
			() => {
				this.timedOut = true;
				this.ws.close();
				this.timedOut = false;
			},
			this.interval
		);

		this.ws.onopen = (event) => {
			clearTimeout(timeout);
			this.state = WebSocket.OPEN;
			reconnect = false;
			//TODO: reopen in case of reconnect ?
			this.reconnects = 0;
			this.keepalive(true);
			this.onopen(event);
		};
		this.ws.onclose = (event) => {
			clearTimeout(timeout);
			this.keepalive(false);

			this.ws = null;
			if (this.closing) {
				this.state = WebSocket.CLOSED;
				this.onclose(event);
			}
			else {
				this.state = WebSocket.CONNECTING;
				if (!reconnect && !this.timedOut) {
					this.onclose(event);
				}
				if (this.reconnects < 10) {
					//TODO: onreconnect event ?
					setTimeout(
						() => {
							this.reconnects++;
							this.connect(true);
						},
						this.reconnectInterval
					);
				}
			}
		};
		this.ws.onmessage = (event) => {
			this.onmessage(event);
		};
		this.ws.onerror = (event) => {
			this.onerror(event);
		};
	}

	keepalive(state) {
		this.aliveTimer && clearTimeout(this.aliveTimer);
		// console.log('keepalive', state);
		if (state) {
			// let self = this;
			let msg = JSON.stringify({ "ping": 1, "tabId": this.tabId });
			this.aliveTimer = setTimeout(
				() => { this.send(msg); },
				10000
			);
		}
		else {
			this.aliveTimer = false;
		}
	}

	send(data) {
		this.keepalive(true);
		// console.log('SEND', data);
		if (this.ws) {
			return this.ws.send(data);
		}
		else {
			throw 'INVALID_STATE_ERR: Pausing to reconnect websocket';
		}
	}

	close() {
		this.keepalive(false);
		this.closing = true;
		if (this.ws) {
			this.ws.close();
		}
	}
}
