import dom from "./Dom.js";

export default class Sortable {

	constructor(tableHeaders, prefix) {
		// console.log('call Sortable::constructor');

		// create table header
		// this.colgroup = dom.colgroup();
		// this.thead = dom.thead();

		this.table = dom.table({
			id: prefix + '-tbl',
			class: "sortable header"
		});
		this.headers = {};

		for (var key in tableHeaders) {
			this.headers[key] = {
				width: tableHeaders[key].width || "auto",
				title: tableHeaders[key].title || key,
				class: tableHeaders[key].class || "listen sortable",
				tooltip: tableHeaders[key].tooltip || '',
				numeric: tableHeaders[key].numeric || true,
				code: tableHeaders[key].code || false,
			}
		}
		// keep headers object
		// this.headers = headers;

		let colgroup = dom.colgroup({appendTo: this.table});
		let thead = dom.thead({ appendTo: this.table });
		this.tbody = dom.tbody({ appendTo: this.table });
		// let tfoot = dom.tfoot({ appendTo: this.table });

		let tr = dom.tr({ appendTo: thead });
		let counter = 0; // counter for easier column number detection

		for (var key in this.headers) {
			dom.col({ style: "width:" + this.headers[key].width, appendTo: colgroup });

			let th = dom.th({ appendTo: tr });
			this.headers[key].btn = dom.div({
				class: this.headers[key].class,
				text: this.headers[key].title,
				title: this.headers[key].tooltip,
				data: {
					action: "sortable",
					col: counter,
					sort: (this.headers[key].numeric ? "number" : "string")
				},
				appendTo: th,
			});
			counter++;
		}
		// console.log('call Sortable::header with columns', counter);

		// footer
		this.fillerRow = dom.tr({ appendTo: this.tbody, style: "height: auto" });
		for (var key in this.headers) {
			dom.td({ class: "bottom-filler", appendTo: this.fillerRow });
		}
	}

	addRows(data) {
		// console.log('Sortable::addRows', data.length);
		for (var i = 0; i < data.length; i++) {
			let row = data[i];
			let tr = dom.tr();

			for (var key in this.headers) {
				let text;
				let tooltip = '';
				if (typeof (row[key]) == 'object') {
					text = row[key].value || "-";
					tooltip = row[key].tooltip || '';
				}
				else {
					text = row[key].toString() || row[key];
				}
				if (isNaN(text) && this.headers[key].numeric) {
					this.headers[key].numeric = false;
					this.headers[key].btn.dataset.sort = "string";
				}
				if (this.headers[key].code) {
					text = '<code>' + text + '</code>';
				}
				dom.td({ html: text, title: tooltip, appendTo: tr });
			}
			this.tbody.insertBefore(tr, this.fillerRow);
		}
	}

	get() {
		return this.table;
	}

	static sort(btn) {
		// console.log('call Sortable::sort');
		let table = false;
		let n = parseInt(btn.dataset['col']);
		let dir = 'asc';

		// find table
		let element = btn.parentNode;
		do {
			if (element.nodeName == 'TABLE') {
				table = element;
				break;
			}
			element = element.parentNode || false;
		}
		while (element);

		if (!table) {
			console.error('Unable to find table: ', btn);
			return;
		}
		// update sorting order icon
		if (btn.classList.contains('desc')) {
			btn.classList.remove('desc');
		}
		else if (btn.classList.contains('asc')) {
			dir = 'desc';
			btn.classList.remove('asc');
		}
		else {
			let btns = table.getElementsByClassName("listen");
			for (var i = 0; i < btns.length; i++) {
				btns[i].classList.remove('desc', 'asc');
			}
		}
		btn.classList.add(dir);

		let tbody = table.querySelector('TBODY');
		let rows = Array.from(tbody.rows);
		let fillerRow = rows.pop();

		rows.sort(
			btn.dataset.sort == 'number'
			? (
				dir == 'asc'
					? (x, y) => { return (parseFloat(x.cells[n].innerText) > parseFloat(y.cells[n].innerText) ? 1 : -1); }
					: (x, y) => { return (parseFloat(x.cells[n].innerText) < parseFloat(y.cells[n].innerText) ? 1 : -1); }
			)
			: (
				dir == 'asc'
					? (x, y) => { return (x.cells[n].innerText > y.cells[n].innerText ? 1 : -1); }
					: (x, y) => { return (x.cells[n].innerText < y.cells[n].innerText ? 1 : -1); }
			)
		);

		rows.push(fillerRow);
		tbody.append(...rows);
	}
}